import { useCallback, useState } from 'react';

/**
 * React hook for combine the state of a value prop and its default value
 **/
export var useControlledState = function useControlledState(controlledValue, defaultValue) {
  var isControlled = controlledValue !== undefined;
  var _useState = useState(isControlled ? controlledValue : defaultValue),
    initialValue = _useState[0];
  var _useState2 = useState(initialValue),
    innerValue = _useState2[0],
    setInnerValue = _useState2[1];
  var updater = useCallback(function (value, forceFlag) {
    if (controlledValue === undefined || forceFlag) {
      setInnerValue(value);
    }
  }, [setInnerValue, controlledValue]);
  return [isControlled ? controlledValue : innerValue, updater, isControlled, initialValue];
};
export default useControlledState;