// Taken from Typescript version in @mountain-ui/react-hooks (https://github.com/tonyghiani/mountain-ui/blob/master/packages/react-hooks/src/useLocalStorage/useLocalStorage.ts)
import { useEffect, useState } from 'react';
import useEventListener from '../useEventListener';
var LOCAL_STORAGE_KEY = 'localStorage';
var hasWindow = function hasWindow() {
  return typeof window !== 'undefined';
};

/**
 *
 * @param key string
 * @param initialValue any
 * @param parserFn func - Function applied in the listener when changing a local storage value manually in the browser
 * @returns {UseLocalStorageResult}
 */
function useLocalStorage(key, initialValue, _temp) {
  var _ref = _temp === void 0 ? {} : _temp,
    _ref$parserFn = _ref.parserFn,
    parserFn = _ref$parserFn === void 0 ? function (value) {
      return value;
    } : _ref$parserFn;
  /**
   * The loadStoredValue retrieve the stored value
   * from the localStorage if it exists.
   * In case the hook is used in SSR, it returns the
   * initial value until it gets hydrated in the client.
   */
  var loadStoredValue = function loadStoredValue() {
    if (!hasWindow()) return initialValue;
    try {
      var item = window.localStorage.getItem(key);
      return item ? JSON.parse(item) : initialValue;
    } catch (error) {
      console.warn("#useLocalStorage: an error occurred loading the localStorage key \u201C" + key + "\u201D:", error);
      return initialValue;
    }
  };

  /**
   * This state holds the value associated with the store key
   * and is served to the component.
   */
  var _useState = useState(loadStoredValue),
    storedValue = _useState[0],
    setStoredValue = _useState[1];

  /**
   * The setValue function check whether is possible
   * to store a new value for the key argument and
   * update the local state to notify the component about the change
   */
  var setValue = function setValue(value) {
    if (!hasWindow()) {
      console.warn("#useLocalStorage: impossible to set the localStorage \u201C" + key + "\u201D inside a no-client context.");
    }
    try {
      var valueToStore = typeof value === 'function' ? value(storedValue) : value;
      var valueToStoreSerialized = JSON.stringify(valueToStore);
      window.localStorage.setItem(key, valueToStoreSerialized);
      setStoredValue(valueToStore);
    } catch (error) {
      console.warn("#useLocalStorage: error setting the localStorage key \u201C" + key + "\u201D:", error);
    }
  };
  useEffect(function () {
    setStoredValue(loadStoredValue());
  }, []);
  useEffect(function () {
    window.dispatchEvent(new CustomEvent(LOCAL_STORAGE_KEY, {
      detail: {
        key: key,
        newValue: storedValue
      }
    }));
  }, [storedValue, key]);
  var handleStorageChange = function handleStorageChange(event) {
    var _event$detail, _event$detail2;
    var eventKey = event.key || ((_event$detail = event.detail) == null ? void 0 : _event$detail.key);
    var eventValue = event.newValue || ((_event$detail2 = event.detail) == null ? void 0 : _event$detail2.newValue);
    if (eventKey === key) setValue(parserFn(eventValue));
  };
  useEventListener(LOCAL_STORAGE_KEY, handleStorageChange);
  useEventListener('storage', handleStorageChange);
  return [storedValue, setValue];
}
export default useLocalStorage;