import _extends from "@babel/runtime/helpers/esm/extends";
import { useEffect, useMemo, useReducer } from 'react';
export var ACTIONS = {
  prev: 'prev',
  next: 'next',
  reset: 'reset'
};
function reducer(state, action) {
  var update = typeof action === 'function' ? action(state) : action;
  var nextState = _extends({}, state, update);
  return _extends({}, nextState, {
    step: nextState.history[nextState.history.length - 1]
  });
}
export default function useSteps(initialStep) {
  var _useReducer = useReducer(reducer, {
      history: [initialStep],
      lastAction: null,
      step: initialStep
    }),
    stepState = _useReducer[0],
    setState = _useReducer[1];
  useEffect(function () {
    setState({
      history: [initialStep]
    });
  }, [initialStep]);

  // Define and memoize the navigation handlers next, prev and reset
  var _useMemo = useMemo(function () {
      return {
        next: function next(nextStep) {
          setState(function (_ref) {
            var history = _ref.history;
            return {
              history: [].concat(history, [nextStep]),
              lastAction: ACTIONS.next
            };
          });
        },
        prev: function prev() {
          setState(function (_ref2) {
            var history = _ref2.history;
            return {
              history: history.slice(0, -1),
              lastAction: ACTIONS.prev
            };
          });
        },
        reset: function reset() {
          setState({
            history: [initialStep],
            lastAction: 'reset'
          });
        }
      };
    }, [initialStep]),
    next = _useMemo.next,
    prev = _useMemo.prev,
    reset = _useMemo.reset;
  var isActive = function isActive(receivedStep) {
    return receivedStep === stepState.step;
  };
  return _extends({
    next: next,
    prev: stepState.history.length > 1 ? prev : null,
    reset: stepState.history.length > 1 ? reset : function () {},
    isActive: isActive
  }, stepState);
}